@@ -3,9 +3,11 @@ package DateTime::Format::Natural::Lang::EN;
 use strict;
 use warnings;
 use base qw(DateTime::Format::Natural::Lang::Base);
-use boolean qw(true false);
+#use boolean qw(true false);
+use constant true  => 1;
+use constant false => 0; 
 
-our $VERSION = '1.15';
+our $VERSION = '1.17';
 
 our (%init,
      %timespan,
@@ -16,6 +18,7 @@ our (%init,
      %data_months,
      %data_months_abbrev,
      @data_months_all,
+     %extended_checks,
      %grammar);
 
 %init = ('tokens' => sub {});
@@ -63,13 +66,27 @@ our (%init,
     $RE{month} = qr/^($months_re)$/i;
 }
 
+%extended_checks = (
+    suffix => sub {
+        my $captured = shift;
+        my @pos = @_;
+        # XXX constant.pm true/false: workaround for a segmentation fault
+        # in Perl_mg_find() on perl 5.8.9 and 5.10.0 when using boolean.pm
+        # v0.20 (tested as of 12/02/2009).
+        return true if ($captured->{$pos[0]} == 1 && $captured->{$pos[1]} !~ /s$/);
+        return true if ($captured->{$pos[0]} >  1 && $captured->{$pos[1]} =~ /s$/);
+        return false;
+    },
+);
+
 # <keyword> => [
 #    [ <PERL TYPE DECLARATION>, ... ], ---------------------> declares how the tokens will be evaluated
 #    [
 #      { <token index> => <token value>, ... }, ------------> declares the index <-> value map
+#      [ [ <index(es) of token(s) to be passed> ], ... ], --> declares which tokens will be passed to the extended check(s) 
 #      [ <subroutine(s) for extended check(s)>, ... ] ------> declares the extended check(s)
-#      [ [ <index(es) of token(s) to be passed> ], ... ], --> declares which tokens will be passed to the dispatch handler(s)
-#      [ <name of subroutine to dispatch to>, ... ], -------> declares the dispatch handler(s)
+#      [ [ <index(es) of token(s) to be passed> ], ... ], --> declares which tokens will be passed to the worker method(s)
+#      [ <name of method to dispatch to>, ... ], -----------> declares the worker method(s)
 #    ],
 
 %grammar = (
@@ -78,6 +95,7 @@ our (%init,
        [
          { 0 => 'now' },
          [],
+         [],
          [ [] ],
          [ '_day_today' ],
        ],
@@ -87,18 +105,21 @@ our (%init,
        [
          { 0 => 'today' },
          [],
+         [],
          [ [] ],
          [ '_day_today' ],
        ],
        [
          { 0 => 'yesterday' },
          [],
+         [],
          [ [] ],
          [ '_day_yesterday' ],
        ],
        [
          { 0 => 'tomorrow' },
          [],
+         [],
          [ [] ],
          [ '_day_tomorrow' ],
        ],
@@ -108,18 +129,21 @@ our (%init,
        [
          { 0 => 'morning' },
          [],
+         [],
          [ [] ],
          [ '_daytime_morning' ],
        ],
        [
          { 0 => 'afternoon' },
          [],
+         [],
          [ [] ],
          [ '_daytime_afternoon' ],
        ],
        [
          { 0 => 'evening' },
          [],
+         [],
          [ [] ],
          [ '_daytime_evening' ],
        ]
@@ -129,12 +153,14 @@ our (%init,
        [
          { 0 => 'noon' },
          [],
+         [],
          [ [] ],
          [ '_daytime_noon' ],
        ],
        [
          { 0 => 'midnight' },
          [],
+         [],
          [ [] ],
          [ '_daytime_midnight' ],
        ],
@@ -144,36 +170,42 @@ our (%init,
        [
          { 0 => 'yesterday', 1 => 'at', 2 => 'noon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_noon' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'at', 2 => 'midnight' },
          [],
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_midnight' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => 'noon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_noon' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => 'midnight' },
          [],
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_midnight' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => 'noon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_noon' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => 'midnight' },
          [],
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_midnight' ],
        ],
@@ -183,18 +215,21 @@ our (%init,
        [
          { 0 => 'this', 1 => 'morning' },
          [],
+         [],
          [ [] ],
          [ '_daytime_morning' ],
        ],
        [
          { 0 => 'this', 1 => 'afternoon' },
          [],
+         [],
          [ [] ],
          [ '_daytime_afternoon' ],
        ],
        [
          { 0 => 'this', 1 => 'evening' },
          [],
+         [],
          [ [] ],
          [ '_daytime_evening' ],
        ],
@@ -204,54 +239,63 @@ our (%init,
        [
          { 0 => 'yesterday', 1 => 'morning' },
          [],
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_morning' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'afternoon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'yesterday', 1 => 'evening' },
          [],
+         [],
          [ [], [] ],
          [ '_day_yesterday', '_daytime_evening' ],
        ],
        [
          { 0 => 'today', 1 => 'morning' },
          [],
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_morning' ],
        ],
        [
          { 0 => 'today', 1 => 'afternoon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'today', 1 => 'evening' },
          [],
+         [],
          [ [], [] ],
          [ '_day_today', '_daytime_evening' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'morning' },
          [],
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_morning' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'afternoon' },
          [],
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_afternoon' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'evening' },
          [],
+         [],
          [ [], [] ],
          [ '_day_tomorrow', '_daytime_evening' ],
        ]
@@ -261,36 +305,42 @@ our (%init,
        [
          { 0 => $RE{time_am}, 1 => 'yesterday' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_yesterday' ],
        ],
        [
          { 0 => $RE{time_am}, 1 => 'today' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_today' ],
        ],
        [
          { 0 => $RE{time_am}, 1 => 'tomorrow' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_time', '_day_tomorrow' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'yesterday' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_yesterday' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'today' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_today' ],
        ],
        [
          { 0 => $RE{time_pm}, 1 => 'tomorrow' },
          [],
+         [],
          [ [ 0 ], [] ],
          [ '_at_pm', '_day_tomorrow' ],
        ],
@@ -300,6 +350,7 @@ our (%init,
        [
          { 0 => $RE{month} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_month' ],
        ],
@@ -309,21 +360,34 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => $RE{month} },
          [],
+         [],
          [ [ 0, 1 ] ],
-         [ '_month_day_before' ],
+         [ '_month_day' ],
        ],
        [
          { 0 => $RE{month}, 1 => $RE{day_enum} },
          [],
-         [ [ 0, 1 ] ],
-         [ '_month_day_after' ],
+         [],
+         [ [ 1, 0 ] ],
+         [ '_month_day' ],
        ]
     ],
+    month_day_year => [
+       [ 'REGEXP', 'REGEXP', 'REGEXP' ],
+       [
+         { 0 => $RE{month}, 1 => $RE{day_enum}, 2 => $RE{year} },
+         [],
+         [],
+         [ [ 1, 0 ], [ 2 ] ],
+         [ '_month_day', '_year' ], 
+       ]
+    ], 
     weekday => [
        [ 'REGEXP' ],
        [
          { 0 => $RE{weekday} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_weekday' ],
        ],
@@ -333,6 +397,7 @@ our (%init,
        [
          { 0 => 'last', 1 => $RE{weekday} },
          [],
+         [],
          [ [ 1 ] ],
          [ '_last_day' ],
        ],
@@ -342,6 +407,7 @@ our (%init,
        [
          { 0 => 'last', 1 => $RE{month} },
          [],
+         [],
          [ [ 1 ] ],
          [ '_last_month' ],
        ],
@@ -351,6 +417,7 @@ our (%init,
        [
          { 0 => 'last', 1 => 'month' },
          [],
+         [],
          [ [] ],
          [ '_last_month_literal' ],
        ],
@@ -360,6 +427,7 @@ our (%init,
        [
          { 0 => 'last', 1 => 'year' },
          [],
+         [],
          [ [] ],
          [ '_last_year' ],
        ],
@@ -369,6 +437,7 @@ our (%init,
        [
          { 0 => 'next', 1 => $RE{weekday} },
          [],
+         [],
          [ [ 1 ] ],
          [ '_next_weekday' ],
        ],
@@ -378,6 +447,7 @@ our (%init,
        [
          { 0 => 'next', 1 => $RE{month} },
          [],
+         [],
          [ [ 1 ] ],
          [ '_next_month' ],
        ],
@@ -387,6 +457,7 @@ our (%init,
        [
          { 0 => 'next', 1 => 'month' },
          [],
+         [],
          [ [] ],
          [ '_next_month_literal' ],
        ],
@@ -396,6 +467,7 @@ our (%init,
        [
          { 0 => 'next', 1 => 'year' },
          [],
+         [],
          [ [] ],
          [ '_next_year' ],
        ],
@@ -405,6 +477,7 @@ our (%init,
        [
          { 0 => 'this', 1 => 'second' },
          [],
+         [],
          [ [] ],
          [ '_this_second' ],
        ],
@@ -414,6 +487,7 @@ our (%init,
        [
          { 0 => 'this', 1 => $RE{weekday} },
          [],
+         [],
          [ [ 1 ] ],
          [ '_this_weekday' ],
        ],
@@ -423,6 +497,7 @@ our (%init,
        [
          { 0 => 'this', 1 => 'month' },
          [],
+         [],
          [ [] ],
          [ '_this_month' ],
        ],
@@ -432,12 +507,14 @@ our (%init,
        [
          { 0 => $RE{time}, 1 => 'am' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_at_am' ],
        ],
        [
          { 0 => $RE{time}, 1 => 'pm' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_at_pm' ],
        ],
@@ -447,12 +524,14 @@ our (%init,
        [
          { 0 => $RE{time_am} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_at_am' ],
        ],
        [
          { 0 => $RE{time_pm} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_at_pm' ],
        ],
@@ -462,6 +541,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => $RE{time} },
          [],
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_time' ],
        ],
@@ -471,6 +551,7 @@ our (%init,
        [
          { 0 => $RE{time} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_time' ],
        ],
@@ -480,6 +561,7 @@ our (%init,
        [
          { 0 => $RE{time_full} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_time_full' ],
        ],
@@ -489,6 +571,7 @@ our (%init,
        [
          { 0 => $RE{month}, 1 => $RE{year} },
          [],
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_month', '_year' ],
        ],
@@ -498,6 +581,7 @@ our (%init,
        [
          { 0 => $RE{year} },
          [],
+         [],
          [ [ 0 ] ],
          [ '_year' ],
        ],
@@ -507,6 +591,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday} },
          [],
+         [],
          [ [ 0, 1 ] ],
          [ '_count_weekday' ],
        ],
@@ -516,6 +601,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => 'day' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_count_yearday' ],
        ],
@@ -525,18 +611,21 @@ our (%init,
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'morning' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_morning' ],
        ],
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'afternoon' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_afternoon' ],
        ],
        [
          { 0 => $RE{number}, 1 => 'in', 2 => 'the', 3 => 'evening' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_daytime_in_the_evening' ],
        ],
@@ -544,44 +633,51 @@ our (%init,
     ago => [
        [ 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
-         { 0 => $RE{number}, 1 => qr/seconds?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(seconds?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_seconds' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(minutes?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_minutes' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_hours' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/days?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(days?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_days' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(weeks?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_weeks' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(months?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_months' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'ago' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(years?)/i, 2 => 'ago' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_ago_years' ],
        ],
@@ -589,44 +685,51 @@ our (%init,
     ago_tomorrow => [
        [ 'SCALAR', 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/seconds?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(seconds?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_seconds' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/minutes?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(minutes?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_minutes' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/hours?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(hours?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_hours' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/days?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(days?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_days' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/weeks?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(weeks?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_weeks' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/months?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(months?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_months' ],
        ],
        [
-         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/years?/i, 3 => 'ago' },
-         [],
+         { 0 => 'tomorrow', 1 => $RE{number}, 2 => qr/(years?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_tomorrow', '_ago_years' ],
        ],
@@ -634,44 +737,51 @@ our (%init,
     ago_yesterday => [
        [ 'REGEXP', 'REGEXP', 'REGEXP', 'SCALAR' ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/seconds?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(seconds?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_seconds' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/minutes?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(minutes?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_minutes' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/hours?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(hours?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_hours' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/days?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(days?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_days' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/weeks?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(weeks?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_weeks' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/months?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(months?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_months' ],
        ],
        [
-         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/years?/i, 3 => 'ago' },
-         [],
+         { 0 => 'yesterday', 1 => $RE{number}, 2 => qr/(years?)/i, 3 => 'ago' },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [], [ 1 ] ],
          [ '_day_yesterday', '_ago_years' ],
        ],
@@ -679,14 +789,16 @@ our (%init,
     weekday_ago_at_time => [
        [ 'REGEXP', 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR', 'REGEXP' ],
        [
-         { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/months?/, 3 => 'ago', 4 => 'at', 5 => $RE{time_am} },
-         [],
+         { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/(months?)/, 3 => 'ago', 4 => 'at', 5 => $RE{time_am} },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 1 ], [ 0 ], [ 5 ] ],
          [ '_ago_months', '_weekday', '_time' ],
        ],
        [
-         { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/months?/, 3 => 'ago', 4 => 'at', 5 => $RE{time_pm} },
-         [],
+         { 0 => $RE{weekday}, 1 => $RE{number}, 2 => qr/(months?)/, 3 => 'ago', 4 => 'at', 5 => $RE{time_pm} },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 1 ], [ 0 ], [ 5 ] ],
          [ '_ago_months', '_weekday', '_at_pm' ],
        ],
@@ -694,74 +806,86 @@ our (%init,
     now_variant => [
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
-         { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(minutes?)/i, 2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_minutes_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/minutes?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(minutes?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_minutes_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_hours_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_hours_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/^days?/i,  2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/^(days?)/i,  2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_days_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/^days?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/^(days?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_days_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(weeks?)/i, 2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_weeks_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/weeks?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(weeks?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_weeks_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(months?)/i, 2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_months_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/months?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(months?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_months_from' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'before', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(years?)/i, 2 => 'before', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_years_before' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/years?/i, 2 => 'from', 3 => 'now' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(years?)/i, 2 => 'from', 3 => 'now' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_now_years_from' ],
        ],
@@ -771,18 +895,21 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'morning' },
          [],
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_morning' ],
        ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'afternoon' },
          [],
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_afternoon' ],
        ],
        [
          { 0 => $RE{weekday}, 1 => $RE{number}, 2 => 'in', 3 => 'the', 4 => 'evening' },
          [],
+         [],
          [ [ 0 ], [ 1 ] ],
          [ '_weekday', '_daytime_in_the_evening' ],
        ],
@@ -793,6 +920,7 @@ our (%init,
        [
          { 0 => 'last', 1 => $RE{weekday}, 2 => 'at', 3 => $RE{time} },
          [],
+         [],
          [ [ 1 ], [ 3 ] ],
          [ '_last_day', '_time' ],
        ],
@@ -802,6 +930,7 @@ our (%init,
        [
          { 0 => 'last', 1 => 'week', 2 => $RE{weekday} },
          [],
+         [],
          [ [ 2 ] ],
          [ '_last_week_day' ],
        ],
@@ -811,6 +940,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => 'last', 2 => 'week' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_day_last_week' ],
        ],
@@ -820,6 +950,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'last', 3 => 'week' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_count_day_last_week' ],
        ],
@@ -829,6 +960,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'last', 3 => 'month' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_count_day_last_month' ],
        ],
@@ -839,6 +971,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => 'next', 2 => 'week' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_weekday_next_week' ],
        ],
@@ -848,6 +981,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => 'day', 2 => 'next', 3 => 'month' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_count_day_next_month' ],
        ],
@@ -857,33 +991,37 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => 'month', 2 => 'next', 3 => 'year' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_count_month_next_year' ],
        ],
     ],
     in_count_minutes => [
-       [ 'SCALAR', 'REGEXP', 'SCALAR' ],
+       [ 'SCALAR', 'REGEXP', 'REGEXP' ],
        [
-         { 0 => 'in', 1 => $RE{number}, 2 => qr/minutes?/i },
-         [],
+         { 0 => 'in', 1 => $RE{number}, 2 => qr/(minutes?)/i },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 1 ] ],
          [ '_in_count_minutes' ],
        ]
     ],
     in_count_hours => [
-       [ 'SCALAR', 'REGEXP', 'SCALAR' ],
+       [ 'SCALAR', 'REGEXP', 'REGEXP' ],
        [
-         { 0 => 'in', 1 => $RE{number}, 2 => 'hours' },
-         [],
+         { 0 => 'in', 1 => $RE{number}, 2 => qr/(hours?)/i },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 1 ] ],
          [ '_in_count_hours' ],
        ],
     ],
     in_count_days => [
-       [ 'SCALAR', 'REGEXP', 'SCALAR' ],
+       [ 'SCALAR', 'REGEXP', 'REGEXP' ],
        [
-         { 0 => 'in', 1 => $RE{number}, 2 => qr/days?/i },
-         [],
+         { 0 => 'in', 1 => $RE{number}, 2 => qr/(days?)/i },
+         [ [ 1, 2 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 1 ] ],
          [ '_in_count_days' ],
        ],
@@ -893,6 +1031,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => 'this', 2 => 'week' },
          [],
+         [],
          [ [ 0 ] ],
          [ '_weekday_this_week' ],
        ],
@@ -902,6 +1041,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday}, 2 => 'this', 3 => $RE{month} },
          [],
+         [],
          [ [ 0, 1, 3 ] ],
          [ '_count_weekday_this_month' ],
        ],
@@ -909,26 +1049,30 @@ our (%init,
     day_variant => [
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'yesterday' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'before', 3 => 'yesterday' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_daytime_variant_before_yesterday' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'tomorrow' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'before', 3 => 'tomorrow' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_daytime_variant_before_tomorrow' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'yesterday' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'after', 3 => 'yesterday' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_daytime_variant_after_yesterday' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'tomorrow' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'after', 3 => 'tomorrow' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_daytime_variant_after_tomorrow' ],
        ],
@@ -936,26 +1080,30 @@ our (%init,
     hourtime => [
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'noon' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'before', 3 => 'noon' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_hourtime_before_noon' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'noon' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'after', 3 => 'noon' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_hourtime_after_noon' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'before', 3 => 'midnight' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'before', 3 => 'midnight' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_hourtime_before_midnight' ],
        ],
        [
-         { 0 => $RE{number}, 1 => qr/hours?/i, 2 => 'after', 3 => 'midnight' },
-         [],
+         { 0 => $RE{number}, 1 => qr/(hours?)/i, 2 => 'after', 3 => 'midnight' },
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0 ] ],
          [ '_hourtime_after_midnight' ],
        ],
@@ -965,18 +1113,21 @@ our (%init,
        [
          { 0 => 'yesterday', 1 => 'at', 2 => $RE{time_am} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_yesterday', '_time' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => $RE{time_am} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_today', '_time' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => $RE{time_am} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_tomorrow', '_time' ],
        ],
@@ -986,6 +1137,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_am} },
          [],
+         [],
          [ [ 0 ], [ 2 ] ],
          [ '_weekday', '_time' ],
        ],
@@ -995,18 +1147,21 @@ our (%init,
        [
          { 0 => 'yesterday', 1 => 'at', 2 => $RE{time_pm} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_yesterday', '_at_pm' ],
        ],
        [
          { 0 => 'today', 1 => 'at', 2 => $RE{time_pm} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_today', '_at_pm' ],
        ],
        [
          { 0 => 'tomorrow', 1 => 'at', 2 => $RE{time_pm} },
          [],
+         [],
          [ [], [ 2 ] ],
          [ '_day_tomorrow', '_at_pm' ],
        ],
@@ -1016,6 +1171,7 @@ our (%init,
        [
          { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_pm} },
          [],
+         [],
          [ [ 0 ], [ 2 ] ],
          [ '_weekday', '_at_pm' ],
        ],
@@ -1025,12 +1181,14 @@ our (%init,
        [
          { 0 => $RE{monthday}, 1 => $RE{month}, 2 => $RE{year} },
          [],
+         [],
          [ [ 0, 1, 2 ] ],
          [ '_day_month_year' ],
        ],
        [
          { 0 => $RE{month}, 1 => $RE{monthday}, 2 => $RE{year} },
          [],
+         [],
          [ [ 1, 0, 2 ] ],
          [ '_day_month_year' ],
        ],
@@ -1040,6 +1198,7 @@ our (%init,
        [
          { 0 => $RE{day_enum}, 1 => $RE{weekday}, 2 => 'in', 3 => $RE{month} },
          [],
+         [],
          [ [ 0, 1, 3 ] ],
          [ '_count_weekday_this_month' ],
        ],
@@ -1048,13 +1207,8 @@ our (%init,
        [ 'REGEXP', 'REGEXP', 'SCALAR', 'SCALAR' ],
        [
          { 0 => $RE{number}, 1 => $RE{weekdays}, 2 => 'from', 3 => 'now' },
-         [ sub {
-             my $captured = shift;
-             return true if ($captured->{0} == 1 && $captured->{1} !~ /s$/);
-             return true if ($captured->{0} >  1 && $captured->{1} =~ /s$/);
-             return false;
-           }
-         ],
+         [ [ 0, 1 ] ],
+         [ $extended_checks{suffix} ],
          [ [ 0, 1 ] ],
          [ '_count_weekday_from_now' ],
        ],
@@ -1236,6 +1390,7 @@ that the parser does not distinguish between lower/upper case):
  11 January
  dec 25
  may 27th
+ march 1st 2009 
  October 2006
  february 14, 2004
  Friday
